//	Distributed under the Boost Software License Version 1.0 https://www.boost.org/LICENSE_1_0.txt
//	Copyright Gero Peterhoff

#ifndef BOOST_QUADMATH_INTEL_HPP
#define BOOST_QUADMATH_INTEL_HPP

#include <boost/config.hpp>



#ifdef __cplusplus
extern "C" {
#endif

#if BOOST_WINDOWS
	//	https://community.intel.com/t5/Intel-C-Compiler/C-Compiler-128-bit-floating-point/m-p/1216949
	typedef _Quad __float128;
#endif

extern __float128 __acosq (__float128);
extern __float128 __acoshq (__float128);
extern __float128 __asinq (__float128);
extern __float128 __asinhq (__float128);
extern __float128 __atanq (__float128);
extern __float128 __atanhq (__float128);
extern __float128 __atan2q (__float128, __float128);
extern __float128 __cbrtq (__float128);
extern __float128 __ceilq (__float128);
extern __float128 __copysignq (__float128, __float128);
extern __float128 __coshq (__float128);
extern __float128 __cosq (__float128);
extern __float128 __erfq (__float128);
extern __float128 __erfcq (__float128);
extern __float128 __exp2q (__float128);
extern __float128 __expq (__float128);
extern __float128 __expm1q (__float128);
extern __float128 __fabsq (__float128);
extern __float128 __fdimq (__float128, __float128);
extern int __finiteq (__float128);
extern __float128 __floorq (__float128);
extern __float128 __fmaq (__float128, __float128, __float128);
extern __float128 __fmaxq (__float128, __float128);
extern __float128 __fminq (__float128, __float128);
extern __float128 __fmodq (__float128, __float128);
extern __float128 __frexpq (__float128, int *);
extern __float128 __hypotq (__float128, __float128);
extern int __isinfq (__float128);
extern int __ilogbq (__float128);
extern int __isnanq (__float128);
extern int __issignalingq (__float128);
extern __float128 __j0q (__float128);
extern __float128 __j1q (__float128);
extern __float128 __jnq (int, __float128);
extern __float128 __ldexpq (__float128, int);
extern __float128 __lgammaq (__float128);
extern long long int __llrintq (__float128);
extern long long int __llroundq (__float128);
extern __float128 __logbq (__float128);
extern __float128 __logq (__float128);
extern __float128 __log10q (__float128);
extern __float128 __log2q (__float128);
extern __float128 __log1pq (__float128);
extern long int __lrintq (__float128);
extern long int __lroundq (__float128);
extern __float128 __modfq (__float128, __float128 *);
extern __float128 __nanq (const char *);
extern __float128 __nearbyintq (__float128);
extern __float128 __nextafterq (__float128, __float128);
extern __float128 __powq (__float128, __float128);
extern __float128 __remainderq (__float128, __float128);
extern __float128 __remquoq (__float128, __float128, int *);
extern __float128 __rintq (__float128);
extern __float128 __roundq (__float128);
extern __float128 __scalblnq (__float128, long int);
extern __float128 __scalbnq (__float128, int);
extern int __signbitq (__float128);
extern void __sincosq (__float128, __float128 *, __float128 *);
extern __float128 __sinhq (__float128);
extern __float128 __sinq (__float128);
extern __float128 __sqrtq (__float128);
extern __float128 __tanq (__float128);
extern __float128 __tanhq (__float128);
extern __float128 __tgammaq (__float128);
extern __float128 __truncq (__float128);
extern __float128 __y0q (__float128);
extern __float128 __y1q (__float128);
extern __float128 __ynq (int, __float128);

#define FLT128_MAX 1.18973149535723176508575932662800702e4932Q
#define FLT128_MIN 3.36210314311209350626267781732175260e-4932Q
#define FLT128_EPSILON 1.92592994438723585305597794258492732e-34Q
#define FLT128_DENORM_MIN 6.475175119438025110924438958227646552e-4966Q
#define FLT128_MANT_DIG 113
#define FLT128_MIN_EXP (-16381)
#define FLT128_MAX_EXP 16384
#define FLT128_DIG 33
#define FLT128_MIN_10_EXP (-4931)
#define FLT128_MAX_10_EXP 4932

#define M_Eq		2.718281828459045235360287471352662498Q  /* e */
#define M_LOG2Eq	1.442695040888963407359924681001892137Q  /* log_2 e */
#define M_LOG10Eq	0.434294481903251827651128918916605082Q  /* log_10 e */
#define M_LN2q		0.693147180559945309417232121458176568Q  /* log_e 2 */
#define M_LN10q		2.302585092994045684017991454684364208Q  /* log_e 10 */
#define M_PIq		3.141592653589793238462643383279502884Q  /* pi */
#define M_PI_2q		1.570796326794896619231321691639751442Q  /* pi/2 */
#define M_PI_4q		0.785398163397448309615660845819875721Q  /* pi/4 */
#define M_1_PIq		0.318309886183790671537767526745028724Q  /* 1/pi */
#define M_2_PIq		0.636619772367581343075535053490057448Q  /* 2/pi */
#define M_2_SQRTPIq	1.128379167095512573896158903121545172Q  /* 2/sqrt(pi) */
#define M_SQRT2q	1.414213562373095048801688724209698079Q  /* sqrt(2) */
#define M_SQRT1_2q	0.707106781186547524400844362104849039Q  /* 1/sqrt(2) */

#ifdef __cplusplus
}
#endif

#endif	//	BOOST_QUADMATH_INTEL_HPP
