//	Distributed under the Boost Software License Version 1.0 https://www.boost.org/LICENSE_1_0.txt
//	Copyright Gero Peterhoff

#ifndef BOOST_QUADMATH_INTEL_MATH_HPP
#define BOOST_QUADMATH_INTEL_MATH_HPP

#include <boost/config.hpp>
#include <boost/quadmath/quadmath_intel.hpp>



#if BOOST_INTEL
namespace std
{
//	basic
inline __float128 abs(__float128 x)								noexcept	{ return __fabsq(x);	}
inline __float128 fabs(__float128 x)							noexcept	{ return __fabsq(x);	}
inline __float128 fmod(__float128 x, __float128 y)				noexcept	{ return __fmodq(x, y);	}
inline __float128 remainder(__float128 x, __float128 y)			noexcept	{ return __remainderq(x, y);	}
inline __float128 remquo(__float128 x, __float128 y, int* quo)	noexcept	{ return __remquoq(x, y, quo);	}
inline __float128 fma(__float128 x, __float128 y, __float128 z)	noexcept	{ return __fmaq(x, y, z);		}
inline __float128 fmax(__float128 x, __float128 y)				noexcept	{ return __fmaxq(x, y);	}
inline __float128 fmin(__float128 x, __float128 y)				noexcept	{ return __fminq(x, y);	}
inline __float128 fdim(__float128 x, __float128 y)				noexcept	{ return __fdimq(x, y);	}
inline __float128 nanq(const char* arg)							noexcept	{ return __nanq(arg);	}

//	exponential
inline __float128 exp(__float128 x)		noexcept	{ return __expq(x);		}
inline __float128 exp2(__float128 x)	noexcept	{ return __exp2q(x);	}
inline __float128 expm1(__float128 x)	noexcept	{ return __expm1q(x);	}

inline __float128 log(__float128 x)		noexcept	{ return __logq(x);		}
inline __float128 log10(__float128 x)	noexcept	{ return __log10q(x);	}
inline __float128 log2(__float128 x)	noexcept	{ return __log2q(x);	}
inline __float128 log1p(__float128 x)	noexcept	{ return __log1pq(x);	}

//	power
inline __float128 pow(__float128 b, __float128 e)	noexcept	{ return __powq(b, e);	}
inline __float128 sqrt(__float128 x)				noexcept	{ return __sqrtq(x);	}
inline __float128 cbrt(__float128 x)				noexcept	{ return __cbrtq(x);	}
inline __float128 hypot(__float128 x, __float128 y)	noexcept	{ return __hypotq(x, y);}
inline __float128 hypot(__float128 x, __float128 y, __float128 z)	noexcept
{
	return __sqrtq(x*x + y*y + z*z);
}

//	trigonometric
inline __float128 sin(__float128 x)		noexcept	{ return __sinq(x);	}
inline __float128 cos(__float128 x)		noexcept	{ return __cosq(x);	}
inline __float128 tan(__float128 x)		noexcept	{ return __tanq(x);	}

inline __float128 asin(__float128 x)	noexcept	{ return __asinq(x);}
inline __float128 acos(__float128 x)	noexcept	{ return __acosq(x);}
inline __float128 atan(__float128 x)	noexcept	{ return __atanq(x);}

inline __float128 atan2(__float128 y, __float128 x)				noexcept	{ return __atan2q(y, x);	}
inline void sincos(__float128 x, __float128* s, __float128* c)	noexcept	{ return __sincosq(x, s, c);}

//	hyperbolic
inline __float128 sinh(__float128 x)	noexcept	{ return __sinhq(x);	}
inline __float128 cosh(__float128 x)	noexcept	{ return __coshq(x);	}
inline __float128 tanh(__float128 x)	noexcept	{ return __tanhq(x);	}

inline __float128 asinh(__float128 x)	noexcept	{ return __asinhq(x);	}
inline __float128 acosh(__float128 x)	noexcept	{ return __acoshq(x);	}
inline __float128 atanh(__float128 x)	noexcept	{ return __atanhq(x);	}

//	error and gamma
inline __float128 erf(__float128 x)		noexcept	{ return __erfq(x);		}
inline __float128 erfc(__float128 x)	noexcept	{ return __erfcq(x);	}
inline __float128 tgamma(__float128 x)	noexcept	{ return __tgammaq(x);	}
inline __float128 lgamma(__float128 x)	noexcept	{ return __lgammaq(x);	}

//	round
inline __float128 ceil(__float128 x)		noexcept	{ return __ceilq(x);	}
inline __float128 floor(__float128 x)		noexcept	{ return __floorq(x);	}
inline __float128 trunc(__float128 x)		noexcept	{ return __truncq(x);	}
inline __float128 round(__float128 x)		noexcept	{ return __roundq(x);	}
inline long int lround(__float128 x)		noexcept	{ return __lroundq(x);	}
inline long long int llround(__float128 x)	noexcept	{ return __llroundq(x);	}
inline __float128 nearbyint(__float128 x)	noexcept	{ return __nearbyintq(x);	}
inline __float128 rint(__float128 x)		noexcept	{ return __rintq(x);	}
inline long int lrint(__float128 x)			noexcept	{ return __lrintq(x);	}
inline long long int llrint(__float128 x)	noexcept	{ return __llrintq(x);	}

//	manipulation
inline __float128 frexp(__float128 x, int* exp)			noexcept	{ return __frexpq(x, exp);	}
inline __float128 ldexp(__float128 x, int exp)			noexcept	{ return __ldexpq(x, exp);	}
inline __float128 modf(__float128 x, __float128* iptr)	noexcept	{ return __modfq(x, iptr);	}
inline __float128 scalbn(__float128 x, int exp)			noexcept	{ return __scalbnq(x, exp);	}
inline __float128 scalbln(__float128 x, long int exp)	noexcept	{ return __scalblnq(x, exp);}
inline int ilogb(__float128 x)							noexcept	{ return __ilogbq(x);		}
inline __float128 logb(__float128 x)					noexcept	{ return __logbq(x);		}
inline __float128 nextafter(__float128 x, __float128 y)	noexcept	{ return __nextafterq(x, y);}
inline __float128 nexttoward(__float128 x, __float128 y)noexcept	{ return -__nextafterq(-x, -y);	}
inline __float128 copysign(__float128 x, __float128 y)	noexcept	{ return __copysignq(x, y);	}

//	classification and comparison
inline int	fpclassify(__float128 x)	noexcept
{
    	 if (__isnanq(x))				return FP_NAN;
    else if (__isinfq(x))				return FP_INFINITE;
    else if (x == 0)					return FP_ZERO;
    else if (__fabs(x) < FLT128_MIN)	return FP_SUBNORMAL;
    else								return FP_NORMAL;
}
inline bool isfinite(__float128 x)		noexcept	{ return __finiteq(x);	}
inline bool isinf(__float128 x)			noexcept	{ return __isinfq(x);	}
inline bool isnan(__float128 x)			noexcept	{ return __isnanq(x);	}
inline bool	isnormal(__float128 x)		noexcept
{
	return fpclassify(x) == FP_NORMAL;
}
inline bool signbit(__float128 x)		noexcept	{ return __signbitq(x);	}
inline bool isunordered(__float128 x, __float128 y)	noexcept
{
	return __isnanq(x) || __isnanq(y);
}
inline bool isgreater(__float128 x, __float128 y)	noexcept
{
	if (BOOST_UNLIKELY(isunordered(x, y)))	return false;
	else									return x > y;
}
inline bool isgreaterequal(__float128 x, __float128 y)	noexcept
{
	if (BOOST_UNLIKELY(isunordered(x, y)))	return false;
	else									return x >= y;
}
inline bool isless(__float128 x, __float128 y)	noexcept
{
	if (BOOST_UNLIKELY(isunordered(x, y)))	return false;
	else									return x < y;
}
inline bool islessequal(__float128 x, __float128 y)	noexcept
{
	if (BOOST_UNLIKELY(isunordered(x, y)))	return false;
	else									return x <= y;
}
inline bool islessgreater(__float128 x, __float128 y)	noexcept
{
	if (BOOST_UNLIKELY(isunordered(x, y)))	return false;
	else									return (x < y) || (x > y);
}

//	more
inline __float128 j0(__float128 x)			noexcept	{ return __j0q(x);		}
inline __float128 j1(__float128 x)			noexcept	{ return __j1q(x);		}
inline __float128 jnq(int x, __float128 y)	noexcept	{ return __jnq(x, y);	}

inline __float128 y0(__float128 x)			noexcept	{ return __y0q(x);		}
inline __float128 y1(__float128 x)			noexcept	{ return __y1q(x);		}
inline __float128 ynq(int x, __float128 y)	noexcept	{ return __ynq(x, y);	}
}	//	std
#endif	//	BOOST_INTEL
#endif	//	BOOST_QUADMATH_INTEL_MATH_HPP
